@@ -23,7 +23,6 @@
 #include <pthread.h>
 #include <sys/stat.h>
 #include <sys/mman.h>
-#include <sys/syscall.h>
 #include <fcntl.h>
 
 #include "common.h"
@@ -357,6 +356,9 @@ lyxml_free_attr(struct ly_ctx *ctx, struct lyxml_elem *parent, struct lyxml_attr
     }
     lydict_remove(ctx, attr->name);
     lydict_remove(ctx, attr->value);
+    if (attr->type == LYXML_ATTR_STD_UNRES) {
+        free((char *)attr->ns);
+    }
     free(attr);
 }
 
@@ -374,6 +376,9 @@ lyxml_free_attrs(struct ly_ctx *ctx, struct lyxml_elem *elem)
 
         lydict_remove(ctx, a->name);
         lydict_remove(ctx, a->value);
+        if (a->type == LYXML_ATTR_STD_UNRES) {
+            free((char *)a->ns);
+        }
         free(a);
 
         a = next;
@@ -809,6 +814,12 @@ parse_attr(struct ly_ctx *ctx, const char *data, unsigned int *len, struct lyxml
                 memcpy(prefix, data, c - data);
                 prefix[c - data] = '\0';
                 attr->ns = lyxml_get_ns(parent, prefix);
+                if (!attr->ns) {
+                    /* remember the prefix for later resolution */
+                    attr->type = LYXML_ATTR_STD_UNRES;
+                    attr->ns = (struct lyxml_ns *)prefix;
+                    prefix = NULL;
+                }
             } else if (((*c == 'm') && (xml_flag == 1)) ||
                     ((*c == 'l') && (xml_flag == 2))) {
                 ++xml_flag;
@@ -1141,6 +1152,16 @@ store_content:
         goto error;
     }
 
+    /* resolve all attribute prefixes */
+    LY_TREE_FOR(elem->attr, attr) {
+        if (attr->type == LYXML_ATTR_STD_UNRES) {
+            str = (char *)attr->ns;
+            attr->ns = lyxml_get_ns(elem, str);
+            free(str);
+            attr->type = LYXML_ATTR_STD;
+        }
+    }
+
     if (!elem->ns && !nons_flag && parent) {
         elem->ns = lyxml_get_ns(parent, prefix_len ? prefix : NULL);
     }
