@@ -1,5 +1,7 @@
 # Copyright (c) 2020 6WIND S.A.
-# SPDX-License-Identifier: MIT
+# SPDX-License-Identifier: BSD-3-Clause
+
+import logging
 
 from _libyang import ffi
 from _libyang import lib
@@ -12,10 +14,14 @@ from .schema import SList
 from .schema import SNode
 from .schema import SRpc
 from .schema import Type
+from .util import LibyangError
 from .util import c2str
 from .util import str2c
 
 
+LOG = logging.getLogger(__name__)
+
+
 #------------------------------------------------------------------------------
 def printer_flags(with_siblings=False, pretty=False, keep_empty_containers=False,
                   trim_default_values=False, include_implicit_defaults=False):
@@ -57,24 +63,37 @@ def path_flags(update=False, rpc_output=False, no_parent_ret=False):
 
 
 #------------------------------------------------------------------------------
-def parser_flags(data=False, config=False, strict=False, trusted=False,
-                 no_yanglib=False):
+def parser_flags(data=False, config=False, get=False, strict=False,
+                 trusted=False, no_yanglib=False, rpc=False, rpcreply=False,
+                 destruct=False, no_siblings=False, explicit=False):
     flags = 0
     if data:
         flags |= lib.LYD_OPT_DATA
     if config:
         flags |= lib.LYD_OPT_CONFIG
+    if get:
+        flags |= lib.LYD_OPT_GET
     if strict:
         flags |= lib.LYD_OPT_STRICT
     if trusted:
         flags |= lib.LYD_OPT_TRUSTED
     if no_yanglib:
         flags |= lib.LYD_OPT_DATA_NO_YANGLIB
+    if rpc:
+        flags |= lib.LYD_OPT_RPC
+    if rpcreply:
+        flags |= lib.LYD_OPT_RPCREPLY
+    if destruct:
+        flags |= lib.LYD_OPT_DESTRUCT
+    if no_siblings:
+        flags |= lib.LYD_OPT_NOSIBLINGS
+    if explicit:
+        flags |= lib.LYD_OPT_EXPLICIT
     return flags
 
 
 #------------------------------------------------------------------------------
-class DNode(object):
+class DNode:
     """
     Data tree node.
     """
@@ -150,17 +169,24 @@ class DNode(object):
         finally:
             lib.free(path)
 
-    def validate(self, data=False, config=False, strict=False, trusted=False,
-                 no_yanglib=False):
+    def validate(self, data=False, config=False, get=False, rpc=False,
+                 rpcreply=False, no_yanglib=False):
         flags = parser_flags(
-            data=data, config=config, strict=strict, trusted=trusted,
-            no_yanglib=no_yanglib)
+            data=data, config=config, get=get, rpc=rpc,
+            rpcreply=rpcreply, no_yanglib=no_yanglib)
         node_p = ffi.new('struct lyd_node **')
         node_p[0] = self._node
         ret = lib.lyd_validate(node_p, flags, ffi.NULL)
         if ret != 0:
             self.context.error('validation failed')
 
+    def merge(self, source, destruct=False, no_siblings=False, explicit=False):
+        flags = parser_flags(destruct=destruct, no_siblings=no_siblings,
+                             explicit=explicit)
+        ret = lib.lyd_merge(self._node, source._node, flags)
+        if ret != 0:
+            raise self.context.error('merge failed')
+
     def print_mem(self, fmt,
                   with_siblings=False,
                   pretty=False,
@@ -230,7 +256,7 @@ class DNode(object):
             keep_empty_containers=keep_empty_containers)
 
         def _to_dict(node, parent_dic):
-            if not lib.lyd_toprint(node._node, flags):
+            if not lib.lyd_node_should_print(node._node, flags):
                 return
             if strip_prefixes:
                 name = node.name()
@@ -262,6 +288,35 @@ class DNode(object):
             _to_dict(dnode, dic)
         return dic
 
+    def merge_data_dict(self, dic, rpc=False, rpcreply=False, strict=False,
+                        data=False, config=False, no_yanglib=False):
+        """
+        Merge a python dictionary into this node. The returned value is the
+        first created node.
+
+        :arg dict dic:
+            The python dictionary to convert.
+        :arg bool rpc:
+            Data represents RPC or action input parameters.
+        :arg bool rpcreply:
+            Data represents RPC or action output parameters.
+        :arg bool strict:
+            Instead of ignoring (with a warning message) data without schema
+            definition, raise an error.
+        :arg bool data:
+            Complete datastore content with configuration as well as state
+            data. To handle possibly missing (but by default required)
+            ietf-yang-library data, use no_yanglib=True.
+        :arg bool config:
+            Complete datastore without state data.
+        :arg bool no_yanglib:
+            Ignore (possibly) missing ietf-yang-library data. Applicable only
+            with data=True.
+        """
+        return dict_to_dnode(dic, self.module(), parent=self,
+                             rpc=rpc, rpcreply=rpcreply, strict=strict,
+                             data=data, config=config, no_yanglib=no_yanglib)
+
     def free(self, with_siblings=True):
         try:
             if with_siblings:
@@ -330,7 +385,7 @@ class DList(DContainer):
 class DLeaf(DNode):
 
     def __init__(self, context, node_p):
-        DNode.__init__(self, context, node_p)
+        super().__init__(context, node_p)
         self._leaf = ffi.cast('struct lyd_node_leaf_list *', node_p)
 
     def value(self):
@@ -358,110 +413,185 @@ class DLeafList(DLeaf):
 
 
 #------------------------------------------------------------------------------
-def dict_to_dnode(dic, schema, parent=None, rpc_input=False, rpc_output=False):
+def dict_to_dnode(dic, module, parent=None, rpc=False, rpcreply=False,
+                  strict=False, data=False, config=False, no_yanglib=False):
     """
-    Convert a python dictionary to a DNode object given a YANG schema object.
-    The returned value is always a top-level data node (i.e.: without parent).
+    Convert a python dictionary to a DNode object given a YANG module object.
+    The return value is the first created node. If parent is not set, a
+    top-level node is returned.
 
     :arg dict dic:
         The python dictionary to convert.
-    :arg SNode or Module schema:
-        The libyang schema object associated with the dictionary. It must be
-        at the same "level" than the dictionary (i.e.: direct children names of
-        the schema object must be keys in the dictionary).
+    :arg Module module:
+        The libyang Module object associated with the dictionary.
     :arg DNode parent:
         Optional parent to update. If not specified a new top-level DNode will
         be created.
-    :arg bool rpc_input:
-        If True, expect schema to be a SRpc object and dic will be parsed
-        by looking in the rpc input nodes.
-    :arg bool rpc_output:
-        If True, expect schema to be a SRpc object and dic will be parsed
-        by looking in the rpc output nodes.
+    :arg bool rpc:
+        Data represents RPC or action input parameters.
+    :arg bool rpcreply:
+        Data represents RPC or action output parameters.
+    :arg bool strict:
+        Instead of ignoring (with a warning message) data without schema
+        definition, raise an error.
+    :arg bool data:
+        Complete datastore content with configuration as well as state
+        data. To handle possibly missing (but by default required)
+        ietf-yang-library data, use no_yanglib=True.
+    :arg bool config:
+        Complete datastore without state data.
+    :arg bool no_yanglib:
+        Ignore (possibly) missing ietf-yang-library data. Applicable only
+        with data=True.
     """
     if not dic:
-        return parent
+        return None
 
     if not isinstance(dic, dict):
         raise TypeError('dic argument must be a python dict')
+    if not isinstance(module, Module):
+        raise TypeError('module argument must be a Module object')
+    if parent is not None and not isinstance(parent, DNode):
+        raise TypeError('parent argument must be a DNode object or None')
 
-    # XXX: ugly, required for python2. Cannot use nonlocal keyword
-    _parent = [parent]
+    created = []
 
-    def _to_dnode(_dic, _schema, key=()):
-        if isinstance(_schema, SRpc):
-            if rpc_input:
-                _schema = _schema.input()
-            elif rpc_output:
-                _schema = _schema.output()
+    def _create_leaf(_parent, module, name, value, in_rpc_output=False):
+        if value is not None:
+            if isinstance(value, bool):
+                value = str(value).lower()
+            elif not isinstance(value, str):
+                value = str(value)
+        if in_rpc_output:
+            n = lib.lyd_new_output_leaf(
+                _parent, module._module, str2c(name), str2c(value))
+        else:
+            n = lib.lyd_new_leaf(
+                _parent, module._module, str2c(name), str2c(value))
+        if not n:
+            if _parent:
+                parent_path = repr(DNode.new(module.context, _parent).path())
             else:
-                raise ValueError('rpc_input or rpc_output must be specified')
-        if not _schema:
-            return
-        for s in _schema:
-            name = s.name()
-            if name not in _dic:
-                name = s.fullname()
-                if name not in _dic:
-                    continue
-            d = _dic[name]
-            if isinstance(s, SContainer):
-                if not isinstance(d, dict):
-                    raise TypeError('%s: python value is not a dict: %r'
-                                    % (s.schema_path(), d))
-                dnode = s.context.create_data_path(
-                    s.data_path() % key, parent=_parent[0],
-                    rpc_output=rpc_output)
-                if _parent[0] is None:
-                    _parent[0] = dnode
-                _to_dnode(d, s, key)
-            elif isinstance(s, SList):
-                if not isinstance(d, (list, tuple)):
-                    raise TypeError('%s: python value is not a list/tuple: %r'
-                                    % (s.schema_path(), d))
-                for element in d:
-                    if not isinstance(element, dict):
-                        raise TypeError('%s: list element is not a dict: %r'
-                                        % (s.schema_path(), element))
-                    try:
-                        next_key = []
-                        for k in s.keys():
-                            try:
-                                next_key.append(element[k.name()])
-                            except KeyError as _e:
-                                try:
-                                    next_key.append(element[k.fullname()])
-                                except KeyError:
-                                    raise _e
-                    except KeyError as e:
-                        raise KeyError(
-                            "%s: key '%s' not present in list element: %r"
-                            % (s.schema_path(), e, element))
-                    _to_dnode(element, s, key + tuple(next_key))
+                parent_path = 'module %r' % module.name()
+            raise module.context.error(
+                'failed to create leaf %r as a child of %s', name, parent_path)
+        created.append(n)
+
+    def _create_container(_parent, module, name, in_rpc_output=False):
+        if in_rpc_output:
+            n = lib.lyd_new_output(_parent, module._module, str2c(name))
+        else:
+            n = lib.lyd_new(_parent, module._module, str2c(name))
+        if not n:
+            if _parent:
+                parent_path = repr(DNode.new(module.context, _parent).path())
+            else:
+                parent_path = 'module %r' % module.name()
+            raise module.context.error(
+                'failed to create container/list/rpc %r as a child of %s',
+                name, parent_path)
+        created.append(n)
+        return n
+
+    schema_cache = {}
+
+    def _find_schema(schema_parent, name, prefix):
+        if isinstance(schema_parent, Module):
+            cache_key = (schema_parent._module, name, prefix)
+        else:
+            cache_key = (schema_parent._node, name, prefix)
+        snode, module = schema_cache.get(cache_key, (None, None))
+        if snode is not None:
+            return snode, module
+        if isinstance(schema_parent, SRpc):
+            if rpc:
+                schema_parent = schema_parent.input()
+            elif rpcreply:
+                schema_parent = schema_parent.output()
+            else:
+                raise ValueError('rpc or rpcreply must be specified')
+            if schema_parent is None:
+                # there may not be any input or any output node in the rpc
+                return None, None
+        for s in schema_parent:
+            if s.name() != name:
+                continue
+            mod = s.module()
+            if prefix is not None and mod.name() != prefix:
+                continue
+            snode = s
+            module = mod
+            break
+        schema_cache[cache_key] = (snode, module)
+        return snode, module
+
+    def _to_dnode(_dic, _schema, _parent=ffi.NULL, in_rpc_output=False):
+        for key, value in _dic.items():
+            if ':' in key:
+                prefix, name = name.split(':')
+            else:
+                prefix, name = None, key
+
+            s, module = _find_schema(_schema, name, prefix)
+            if not s:
+                if isinstance(_schema, Module):
+                    path = _schema.name()
+                elif isinstance(_schema, SNode):
+                    path = _schema.schema_path()
+                else:
+                    path = str(_schema)
+                if strict:
+                    raise LibyangError('%s: unknown element %r' % (path, key))
+                LOG.warning('%s: skipping unknown element %r', path, key)
+                continue
+
+            if isinstance(s, SLeaf):
+                _create_leaf(_parent, module, name, value, in_rpc_output)
+
             elif isinstance(s, SLeafList):
-                if not isinstance(d, (list, tuple)):
+                if not isinstance(value, (list, tuple)):
                     raise TypeError('%s: python value is not a list/tuple: %r'
-                                    % (s.schema_path(), d))
-                for element in d:
-                    dnode = s.context.create_data_path(
-                        s.data_path() % key, parent=_parent[0],
-                        value=element, rpc_output=rpc_output)
-                    if _parent[0] is None:
-                        _parent[0] = dnode
-            elif isinstance(s, SLeaf):
-                dnode = s.context.create_data_path(
-                    s.data_path() % key, parent=_parent[0], value=d,
-                    rpc_output=rpc_output)
-                if _parent[0] is None:
-                    _parent[0] = dnode
+                                    % (s.schema_path(), value))
+                for v in value:
+                    _create_leaf(_parent, module, name, v, in_rpc_output)
 
-    _to_dnode(dic, schema)
+            elif isinstance(s, SRpc):
+                n = _create_container(_parent, module, name, in_rpc_output)
+                _to_dnode(value, s, n, rpcreply)
 
-    # XXX: ugly, required for python2 support
-    parent = _parent[0]
+            elif isinstance(s, SContainer):
+                n = _create_container(_parent, module, name, in_rpc_output)
+                _to_dnode(value, s, n, in_rpc_output)
 
-    if parent is not None:
-        # go back to the root of the created tree
-        parent = parent.root()
+            elif isinstance(s, SList):
+                if not isinstance(value, (list, tuple)):
+                    raise TypeError('%s: python value is not a list/tuple: %r'
+                                    % (s.schema_path(), value))
+                for v in value:
+                    if not isinstance(v, dict):
+                        raise TypeError('%s: list element is not a dict: %r'
+                                        % (_schema.schema_path(), v))
+                    n = _create_container(_parent, module, name, in_rpc_output)
+                    _to_dnode(v, s, n, in_rpc_output)
 
-    return parent
+    result = None
+
+    try:
+        if parent is not None:
+            _parent = parent._node
+            _schema_parent = parent.schema()
+        else:
+            _parent = ffi.NULL
+            _schema_parent = module
+        _to_dnode(dic, _schema_parent, _parent,
+                  in_rpc_output=rpcreply and isinstance(parent, DRpc))
+        if created:
+            result = DNode.new(module.context, created[0])
+            result.validate(rpc=rpc, rpcreply=rpc,
+                            data=data, config=config, no_yanglib=no_yanglib)
+    except:
+        for c in reversed(created):
+            lib.lyd_free(c)
+        raise
+
+    return result
