@@ -1,5 +1,5 @@
 # Copyright (c) 2018-2019 Robin Jarry
-# SPDX-License-Identifier: MIT
+# SPDX-License-Identifier: BSD-3-Clause
 
 from _libyang import ffi
 from _libyang import lib
@@ -33,7 +33,7 @@ def schema_out_format(fmt_string):
 
 
 #------------------------------------------------------------------------------
-class Module(object):
+class Module:
 
     def __init__(self, context, module_p):
         self.context = context
@@ -117,8 +117,8 @@ class Module(object):
         if ret != 0:
             raise self.context.error('cannot print module')
 
-    def parse_data_dict(self, dic, parent=None,
-                        rpc_input=False, rpc_output=False):
+    def parse_data_dict(self, dic, rpc=False, rpcreply=False, strict=False,
+                        data=False, config=False, no_yanglib=False):
         """
         Convert a python dictionary to a DNode object following the schema of
         this module. The returned value is always a top-level data node (i.e.:
@@ -126,21 +126,31 @@ class Module(object):
 
         :arg dict dic:
             The python dictionary to convert.
-        :arg DNode parent:
-            Optional parent to update. If not specified a new top-level DNode
-            will be created.
-        :arg bool rpc_input:
-            If True, dic will be parsed by looking in the rpc input nodes.
-        :arg bool rpc_output:
-            If True, dic will be parsed by looking in the rpc output nodes.
+        :arg bool rpc:
+            Data represents RPC or action input parameters.
+        :arg bool rpcreply:
+            Data represents RPC or action output parameters.
+        :arg bool strict:
+            Instead of ignoring (with a warning message) data without schema
+            definition, raise an error.
+        :arg bool data:
+            Complete datastore content with configuration as well as state
+            data. To handle possibly missing (but by default required)
+            ietf-yang-library data, use no_yanglib=True.
+        :arg bool config:
+            Complete datastore without state data.
+        :arg bool no_yanglib:
+            Ignore (possibly) missing ietf-yang-library data. Applicable only
+            with data=True.
         """
         from .data import dict_to_dnode  # circular import
-        return dict_to_dnode(dic, self, parent=parent,
-                             rpc_input=rpc_input, rpc_output=rpc_output)
+        return dict_to_dnode(dic, self, parent=None,
+                             rpc=rpc, rpcreply=rpcreply, strict=strict,
+                             data=data, config=config, no_yanglib=no_yanglib)
 
 
 #------------------------------------------------------------------------------
-class Revision(object):
+class Revision:
 
     def __init__(self, context, rev_p):
         self.context = context
@@ -160,11 +170,14 @@ class Revision(object):
             yield Extension(self.context, self._rev.ext[i])
 
     def get_extension(self, name, prefix=None, arg_value=None):
-        ext = lib.lypy_find_ext(
-            self._rev.ext, self._rev.ext_size,
-            str2c(name), str2c(prefix), str2c(arg_value))
-        if ext:
-            return Extension(self.context, ext)
+        for ext in self.extensions():
+            if ext.name() != name:
+                continue
+            if prefix is not None and ext.module().name() != prefix:
+                continue
+            if arg_value is not None and ext.argument() != arg_value:
+                continue
+            return ext
         return None
 
     def __repr__(self):
@@ -176,7 +189,7 @@ class Revision(object):
 
 
 #------------------------------------------------------------------------------
-class Extension(object):
+class Extension:
 
     def __init__(self, context, ext_p):
         self.context = context
@@ -204,7 +217,7 @@ class Extension(object):
 
 
 #------------------------------------------------------------------------------
-class Type(object):
+class Type:
 
     DER = lib.LY_TYPE_DER
     BINARY = lib.LY_TYPE_BINARY
@@ -255,15 +268,12 @@ class Type(object):
 
     def get_bases(self):
         if self._type.base == lib.LY_TYPE_DER:
-            for b in self.derived_type().get_bases():
-                yield b
+            yield from self.derived_type().get_bases()
         elif self._type.base == lib.LY_TYPE_LEAFREF:
-            for b in self.leafref_type().get_bases():
-                yield b
+            yield from self.leafref_type().get_bases()
         elif self._type.base == lib.LY_TYPE_UNION:
             for t in self.union_types():
-                for b in t.get_bases():
-                    yield b
+                yield from t.get_bases()
         else:  # builtin type
             yield self
 
@@ -323,8 +333,7 @@ class Type(object):
 
     def all_enums(self):
         for b in self.get_bases():
-            for e in b.enums():
-                yield e
+            yield from b.enums()
 
     def bits(self):
         if self._type.base != self.BITS:
@@ -338,8 +347,7 @@ class Type(object):
 
     def all_bits(self):
         for b in self.get_bases():
-            for bb in b.bits():
-                yield bb
+            yield from b.bits()
 
     NUM_TYPES = frozenset(
         (INT8, INT16, INT32, INT64, UINT8, UINT16, UINT32, UINT64))
@@ -356,8 +364,7 @@ class Type(object):
     def all_ranges(self):
         if self._type.base == lib.LY_TYPE_UNION:
             for t in self.union_types():
-                for r in t.all_ranges():
-                    yield r
+                yield from t.all_ranges()
         else:
             rng = self.range()
             if rng is not None:
@@ -375,8 +382,7 @@ class Type(object):
     def all_lengths(self):
         if self._type.base == lib.LY_TYPE_UNION:
             for t in self.union_types():
-                for l in t.all_lengths():
-                    yield l
+                yield from t.all_lengths()
         else:
             length = self.length()
             if length is not None:
@@ -398,17 +404,14 @@ class Type(object):
             #     ('[xX][mM][lL].*', True)
             yield c2str(p.expr + 1), invert_match
         if self._type.der:
-            for p in self.derived_type().patterns():
-                yield p
+            yield from self.derived_type().patterns()
 
     def all_patterns(self):
         if self._type.base == lib.LY_TYPE_UNION:
             for t in self.union_types():
-                for p in t.all_patterns():
-                    yield p
+                yield from t.all_patterns()
         else:
-            for p in self.patterns():
-                yield p
+            yield from self.patterns()
 
     def module(self):
         module_p = lib.lys_main_module(self._type.der.module)
@@ -424,15 +427,14 @@ class Type(object):
                 yield Extension(self.context, self._type.parent.ext[i])
 
     def get_extension(self, name, prefix=None, arg_value=None):
-        ext = lib.lypy_find_ext(
-            self._type.ext, self._type.ext_size,
-            str2c(name), str2c(prefix), str2c(arg_value))
-        if not ext and self._type.parent:
-            ext = lib.lypy_find_ext(
-                self._type.parent.ext, self._type.parent.ext_size,
-                str2c(name), str2c(prefix), str2c(arg_value))
-        if ext:
-            return Extension(self.context, ext)
+        for ext in self.extensions():
+            if ext.name() != name:
+                continue
+            if prefix is not None and ext.module().name() != prefix:
+                continue
+            if arg_value is not None and ext.argument() != arg_value:
+                continue
+            return ext
         return None
 
     def __repr__(self):
@@ -444,7 +446,7 @@ class Type(object):
 
 
 #------------------------------------------------------------------------------
-class Feature(object):
+class Feature:
 
     def __init__(self, context, feature_p):
         self.context = context
@@ -483,7 +485,7 @@ class Feature(object):
 
 
 #------------------------------------------------------------------------------
-class IfFeatureExpr(object):
+class IfFeatureExpr:
 
     def __init__(self, context, iffeature_p):
         self.context = context
@@ -537,7 +539,7 @@ class IfFeatureExpr(object):
 
 
 #------------------------------------------------------------------------------
-class IfFeatureExprTree(object):
+class IfFeatureExprTree:
 
     def dump(self, indent=0):
         raise NotImplementedError()
@@ -615,7 +617,7 @@ class IfOrFeatures(IfFeatureExprTree):
 
 
 #------------------------------------------------------------------------------
-class SNode(object):
+class SNode:
 
     CONTAINER = lib.LYS_CONTAINER
     LEAF = lib.LYS_LEAF
@@ -648,11 +650,7 @@ class SNode(object):
         return c2str(self._node.name)
 
     def fullname(self):
-        try:
-            s = lib.lypy_node_fullname(self._node)
-            return c2str(s)
-        finally:
-            lib.free(s)
+        return '%s:%s' % (self.module().name(), self.name())
 
     def description(self):
         return c2str(self._node.dsc)
@@ -692,9 +690,9 @@ class SNode(object):
         finally:
             lib.free(s)
 
-    def data_path(self):
+    def data_path(self, key_placeholder="'%s'"):
         try:
-            s = lib.lypy_data_path_pattern(self._node)
+            s = lib.lys_data_path_pattern(self._node, str2c(key_placeholder))
             return c2str(s)
         finally:
             lib.free(s)
@@ -704,11 +702,14 @@ class SNode(object):
             yield Extension(self.context, self._node.ext[i])
 
     def get_extension(self, name, prefix=None, arg_value=None):
-        ext = lib.lypy_find_ext(
-            self._node.ext, self._node.ext_size,
-            str2c(name), str2c(prefix), str2c(arg_value))
-        if ext:
-            return Extension(self.context, ext)
+        for ext in self.extensions():
+            if ext.name() != name:
+                continue
+            if prefix is not None and ext.module().name() != prefix:
+                continue
+            if arg_value is not None and ext.argument() != arg_value:
+                continue
+            return ext
         return None
 
     def if_features(self):
@@ -730,27 +731,6 @@ class SNode(object):
     def __str__(self):
         return self.name()
 
-    def parse_data_dict(self, dic, parent=None,
-                        rpc_input=False, rpc_output=False):
-        """
-        Convert a python dictionary to a DNode object following the schema of
-        this module. The returned value is always a top-level data node (i.e.:
-        without parent).
-
-        :arg dict dic:
-            The python dictionary to convert.
-        :arg DNode parent:
-            Optional parent to update. If not specified a new top-level DNode
-            will be created.
-        :arg bool rpc_input:
-            If True, dic will be parsed by looking in the rpc input nodes.
-        :arg bool rpc_output:
-            If True, dic will be parsed by looking in the rpc output nodes.
-        """
-        from .data import dict_to_dnode  # circular import
-        return dict_to_dnode(dic, self, parent=parent,
-                             rpc_input=rpc_input, rpc_output=rpc_output)
-
     NODETYPE_CLASS = {}
 
     @classmethod
@@ -771,7 +751,7 @@ class SNode(object):
 class SLeaf(SNode):
 
     def __init__(self, context, node_p):
-        SNode.__init__(self, context, node_p)
+        super().__init__(context, node_p)
         self._leaf = ffi.cast('struct lys_node_leaf *', node_p)
 
     def default(self):
@@ -801,7 +781,7 @@ class SLeaf(SNode):
 class SLeafList(SNode):
 
     def __init__(self, context, node_p):
-        SNode.__init__(self, context, node_p)
+        super().__init__(context, node_p)
         self._leaflist = ffi.cast('struct lys_node_leaflist *', node_p)
 
     def ordered(self):
@@ -830,7 +810,7 @@ class SLeafList(SNode):
 class SContainer(SNode):
 
     def __init__(self, context, node_p):
-        SNode.__init__(self, context, node_p)
+        super().__init__(context, node_p)
         self._container = ffi.cast('struct lys_node_container *', node_p)
 
     def presence(self):
@@ -852,7 +832,7 @@ class SContainer(SNode):
 class SList(SNode):
 
     def __init__(self, context, node_p):
-        SNode.__init__(self, context, node_p)
+        super().__init__(context, node_p)
         self._list = ffi.cast('struct lys_node_list *', node_p)
 
     def ordered(self):
