@@ -1,5 +1,5 @@
 # Copyright (c) 2018-2019 Robin Jarry
-# SPDX-License-Identifier: MIT
+# SPDX-License-Identifier: BSD-3-Clause
 
 from _libyang import ffi
 
@@ -10,7 +10,7 @@ class LibyangError(Exception):
 
 
 #------------------------------------------------------------------------------
-def str2c(s):
+def str2c(s, encode=True):
     if s is None:
         return ffi.NULL
     if hasattr(s, 'encode'):
@@ -19,7 +19,7 @@ def str2c(s):
 
 
 #------------------------------------------------------------------------------
-def c2str(c):
+def c2str(c, decode=True):
     if c == ffi.NULL:
         return None
     s = ffi.string(c)
